@@ -1,5 +1,5 @@
-`lordif` <-
-function(resp.data,group,selection=NULL,criterion="Chisqr",pseudo.R2="McFadden",alpha=0.01,beta.change=0.1,R2.change=0.035,
+lordif <-
+function(resp.data,group,selection=NULL,criterion="Chisqr",pseudo.R2="McFadden",alpha=0.01,beta.change=0.1,R2.change=0.02,
               maxIter=10,minCell=5,minTheta=-4.0,maxTheta=4.0,inc=0.1) {
 if (!(toupper(criterion) %in% c("CHISQR","R2","BETA"))){warning("criterion must be one of the following: \"Chisqr\", \"R2\", or \"Beta\", will be reset to default");criterion<-"Chisqr"}
 if (!(toupper(pseudo.R2) %in% c("MCFADDEN","NAGELKERKE","COXSNELL"))) {warning("pseudo.R2 must be one of the following \"McFadden\", \"Nagelkerke\", or \"CoxSnell\", will be reset to default");pseudo.R2<-"McFadden"}
@@ -51,7 +51,7 @@ warning("all items got flagged for DIF - stopping\n")
 }
 sparse.matrix<-NULL;calib.sparse<-NULL;ipar.sparse<-NULL;theta.sparse<-NULL
 if (!compare(flags,flag.matrix)) {
-while (!compare(flags,flag.matrix) & iter < maxIter) {
+repeat {
 iter<-iter+1
 flag.matrix<-rbind(flag.matrix,flags)
 sparse.matrix<-separate(resp.recoded,flags,group)
@@ -61,6 +61,7 @@ eqconst<-equate(ipar[!flags,],ipar.sparse[1:sum(!flags),],theta.grid)
 ipar.sparse[,1]<-ipar.sparse[,1]/eqconst[1]
 ipar.sparse[,2:ncol(ipar.sparse)]<-ipar.sparse[,2:ncol(ipar.sparse)]*eqconst[1]+eqconst[2]
 theta.sparse<-calctheta(ipar.sparse,sparse.matrix,theta.grid)
+pre.flags<-flags
 out<-rundif(selection,resp.recoded,theta.sparse$theta,group,criterion,alpha,beta.change,pseudo.R2,R2.change)
 flags<-out$flag
 ndif<-sum(flags)
@@ -69,6 +70,18 @@ if (ndif==ni) {
 warning("all items got flagged for DIF - stopping\n")
 break
 }
+if (compare(flags,flag.matrix) | iter == maxIter) {
+if (!all(pre.flags==flags)) {
+sparse.matrix<-separate(resp.recoded,flags,group)
+calib.sparse<-grm(sparse.matrix,constrained=FALSE,IRT.param=TRUE,control=list(iter.qN=50)) 
+ipar.sparse<-extract(calib.sparse) 
+eqconst<-equate(ipar[!flags,],ipar.sparse[1:sum(!flags),],theta.grid)
+ipar.sparse[,1]<-ipar.sparse[,1]/eqconst[1]
+ipar.sparse[,2:ncol(ipar.sparse)]<-ipar.sparse[,2:ncol(ipar.sparse)]*eqconst[1]+eqconst[2]
+theta.sparse<-calctheta(ipar.sparse,sparse.matrix,theta.grid)
+}
+break
+}
 }
 if (!compare(flags,flag.matrix) & (iter == maxIter)) {
 sparse.matrix<-separate(resp.recoded,flags,group)
@@ -86,3 +99,4 @@ return(list(options=list(criterion=criterion,pseudo.R2=pseudo.R2,alpha=alpha,bet
               flag=out$flag,recoded=resp.recoded,group=group,ng=ng,ni=ni,ncat=ncat,calib=theta,calib.sparse=theta.sparse,
               iteration=iter,ipar=ipar,ipar.sparse=ipar.sparse,stats.raw=outraw$stats,meanraw=meanraw,flag.raw=outraw$flag))
 }
+
